@@ -2,6 +2,7 @@
  * Copyright (c) 2004-2009 Voltaire, Inc. All rights reserved.
  * Copyright (c) 2002-2012 Mellanox Technologies LTD. All rights reserved.
  * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
+ * Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -158,14 +159,14 @@ typedef struct osm_mgrp_box {
 *		The host ordered LID of this Multicast Group (must be
 *		>= 0xC000).
 *
-*	p_root
+*	mgrp_list
+*		List of multicast groups (mpgr object) having same MLID value.
+*
+*	root
 *		Pointer to the root "tree node" in the single spanning tree
 *		for this multicast group.  The nodes of the tree represent
 *		switches.  Member ports are not represented in the tree.
 *
-*	mgrp_list
-*		List of multicast groups (mpgr object) having same MLID value.
-*
 * SEE ALSO
 *********/
 
@@ -447,7 +448,7 @@ void osm_mgrp_delete_port(IN osm_subn_t * subn, IN osm_log_t * log,
 * SEE ALSO
 *********/
 
-void osm_mgrp_remove_port(osm_subn_t * subn, osm_log_t * log, osm_mgrp_t * mgrp,
+boolean_t osm_mgrp_remove_port(osm_subn_t * subn, osm_log_t * log, osm_mgrp_t * mgrp,
 			  osm_mcm_alias_guid_t * mcm_alias_guid,
 			  ib_member_rec_t * mcmr);
 void osm_mgrp_cleanup(osm_subn_t * subn, osm_mgrp_t * mpgr);
