@@ -2,6 +2,7 @@
  * Copyright (c) 2004-2009 Voltaire, Inc. All rights reserved.
  * Copyright (c) 2002-2011 Mellanox Technologies LTD. All rights reserved.
  * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
+ * Copyright (C) 2012-2017 Tokyo Institute of Technology. All rights reserved.
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -440,6 +441,39 @@ static inline void osm_physp_set_health(IN osm_physp_t * p_physp,
 *	Port, Physical Port
 *********/
 
+/****f* OpenSM: Physical Port/osm_link_is_throttled
+* NAME
+*	osm_link_is_throttled
+*
+* DESCRIPTION
+*	Returns TRUE if the link speed/width given by the physical port is
+*	below the theoretical maximum enabled by both ports, and FALSE
+*	otherwise.
+*
+* SYNOPSIS
+*/
+boolean_t osm_link_is_throttled(IN osm_physp_t * p_physp,
+				IN const boolean_t subn_has_fdr10_enabled);
+/*
+* PARAMETERS
+*	p_physp
+*		[in] Pointer to an osm_physp_t object.
+*
+*	subn_has_fdr10_enabled
+*		[in] TRUE if FDR10 is enabled for subnet, or FALSE otherwise.
+*
+* RETURN VALUES
+*	FALSE if both directions of the link have active link speed/width equal
+*	to the common maximum which both sides have enabled, and TRUE otherwise.
+*	The return value is independent of whether or not the link was
+*	throttled intentionally by the admin or throttled automatically by the
+*	link auto-negotiation.
+*
+* NOTES
+*
+* SEE ALSO
+*********/
+
 /****f* OpenSM: Physical Port/osm_physp_set_port_info
 * NAME
 *	osm_physp_set_port_info
@@ -1611,5 +1645,91 @@ void osm_alias_guid_delete(IN OUT osm_alias_guid_t ** pp_alias_guid);
 *	Port
 *********/
 
+/****f* OpenSM: Port/osm_alias_guid_get_alias_guid
+* NAME
+*	osm_alias_guid_get_alias_guid
+*
+* DESCRIPTION
+*	This function retrieves alias guid from alias guid object.
+*
+* SYNOPSIS
+*/
+static inline ib_net64_t osm_alias_guid_get_alias_guid(IN osm_alias_guid_t *p_alias_guid)
+{
+	CL_ASSERT(p_alias_guid);
+	return p_alias_guid->alias_guid;
+}
+/*
+* PARAMETERS
+*	p_alias_guid
+*		[in] Pointer to a pointer to an alias guid object.
+*
+* RETURN VALUE
+*	This function returns the alias guid or NULL if fails.
+*
+* NOTES
+*
+* SEE ALSO
+*	Port
+*********/
+
+/****f* OpenSM: Port/osm_alias_guid_get_base_guid
+* NAME
+*	osm_alias_guid_get_base_guid
+*
+* DESCRIPTION
+*	This function retrieves base guid from alias guid object.
+*
+* SYNOPSIS
+*/
+static inline ib_net64_t osm_alias_guid_get_base_guid(IN osm_alias_guid_t *p_alias_guid)
+{
+	CL_ASSERT(p_alias_guid);
+	return osm_port_get_guid(p_alias_guid->p_base_port);
+}
+/*
+* PARAMETERS
+*	p_alias_guid
+*		[in] Pointer to a pointer to an alias guid object.
+*
+* RETURN VALUE
+*	This function returns the base guid or NULL if fails.
+*
+* NOTES
+*
+* SEE ALSO
+*	Port
+*********/
+
+/****f* OpenSM: Port/osm_port_clear_base_lid
+* NAME
+*	osm_port_clear_base_lid
+*
+* DESCRIPTION
+*	Clear the base lid of a given port.
+*
+* SYNOPSIS
+*/
+static inline void osm_port_clear_base_lid(OUT osm_port_t * p_port)
+{
+	CL_ASSERT(p_port);
+	p_port->lid = 0;
+	osm_physp_set_base_lid(p_port->p_physp, 0);
+}
+
+/*
+* PARAMETERS
+*	p_port
+*		[out] Pointer to a Port object.
+*
+* RETURN VALUE
+*	None
+*
+* NOTES
+*
+* SEE ALSO
+*	Port
+*********/
+
 END_C_DECLS
 #endif				/* _OSM_PORT_H_ */
