@@ -190,15 +190,21 @@ static osm_madw_t *get_madw(osm_vendor_t * p_vend, ib_net64_t * tid,
 	return 0;
 }
 
+/*
+ * If match table full, evict LRU (least recently used) transaction.
+ * Maintain 2 LRUs: one for SMPs, and one for others (GS).
+ * Evict LRU GS transaction if one is available and only evict LRU SMP
+ * transaction if no other choice.
+ */
 static void
 put_madw(osm_vendor_t * p_vend, osm_madw_t * p_madw, ib_net64_t tid,
 	 uint8_t mgmt_class)
 {
-	umad_match_t *m, *e, *old_lru, *lru = 0;
+	umad_match_t *m, *e, *old_lru, *lru = 0, *lru_smp = 0;
 	osm_madw_t *p_req_madw;
 	osm_umad_bind_info_t *p_bind;
 	ib_net64_t old_tid;
-	uint32_t oldest = ~0;
+	uint32_t oldest = ~0, oldest_smp = ~0;
 	uint8_t old_mgmt_class;
 
 	pthread_mutex_lock(&p_vend->match_tbl_mutex);
@@ -213,15 +219,30 @@ put_madw(osm_vendor_t * p_vend, osm_madw_t * p_madw, ib_net64_t tid,
 			pthread_mutex_unlock(&p_vend->match_tbl_mutex);
 			return;
 		}
-		if (oldest >= m->version) {
-			oldest = m->version;
-			lru = m;
+		if (m->mgmt_class == IB_MCLASS_SUBN_DIR ||
+		    m->mgmt_class == IB_MCLASS_SUBN_LID) {
+			if (oldest_smp >= m->version) {
+				oldest_smp = m->version;
+				lru_smp = m;
+			}
+		} else {
+			if (oldest >= m->version) {
+				oldest = m->version;
+				lru = m;
+			}
 		}
 	}
 
-	old_lru = lru;
-	old_tid = lru->tid;
-	old_mgmt_class = lru->mgmt_class;
+	if (oldest != ~0) {
+		old_lru = lru;
+		old_tid = lru->tid;
+		old_mgmt_class = lru->mgmt_class;
+	} else {
+		CL_ASSERT(oldest_smp != ~0);
+		old_lru = lru_smp;
+		old_tid = lru_smp->tid;
+		old_mgmt_class = lru_smp->mgmt_class;
+	}
 	p_req_madw = old_lru->v;
 	p_bind = p_req_madw->h_bind;
 	p_req_madw->status = IB_CANCELED;
@@ -229,11 +250,20 @@ put_madw(osm_vendor_t * p_vend, osm_madw_t * p_madw, ib_net64_t tid,
 	pthread_mutex_lock(&p_vend->cb_mutex);
 	(*p_bind->send_err_callback) (p_bind->client_context, p_req_madw);
 	pthread_mutex_unlock(&p_vend->cb_mutex);
-	lru->tid = tid;
-	lru->mgmt_class = mgmt_class;
-	lru->v = p_madw;
-	lru->version =
-	    cl_atomic_inc((atomic32_t *) & p_vend->mtbl.last_version);
+	if (mgmt_class == IB_MCLASS_SUBN_DIR ||
+	    mgmt_class == IB_MCLASS_SUBN_LID) {
+		lru_smp->tid = tid;
+		lru_smp->mgmt_class = mgmt_class;
+		lru_smp->v = p_madw;
+		lru_smp->version =
+		    cl_atomic_inc((atomic32_t *) & p_vend->mtbl.last_version);
+	} else {
+		lru->tid = tid;
+		lru->mgmt_class = mgmt_class;
+		lru->v = p_madw;
+		lru->version =
+		    cl_atomic_inc((atomic32_t *) & p_vend->mtbl.last_version);
+	}
 	pthread_mutex_unlock(&p_vend->match_tbl_mutex);
 	OSM_LOG(p_vend->p_log, OSM_LOG_ERROR, "ERR 5402: "
 		"evicting entry %p (tid was 0x%" PRIx64
@@ -246,9 +276,10 @@ ib_mad_addr_conv(ib_user_mad_t * umad, osm_mad_addr_t * osm_mad_addr,
 		 int is_smi)
 {
 	ib_mad_addr_t *ib_mad_addr = umad_get_mad_addr(umad);
+
+	memset(osm_mad_addr, 0, sizeof(osm_mad_addr_t));
 	osm_mad_addr->dest_lid = ib_mad_addr->lid;
 	osm_mad_addr->path_bits = ib_mad_addr->path_bits;
-	osm_mad_addr->static_rate = 0;
 
 	if (is_smi) {
 		osm_mad_addr->addr_type.smi.source_lid = osm_mad_addr->dest_lid;
@@ -260,9 +291,16 @@ ib_mad_addr_conv(ib_user_mad_t * umad, osm_mad_addr_t * osm_mad_addr,
 	osm_mad_addr->addr_type.gsi.remote_qkey = ib_mad_addr->qkey;
 	osm_mad_addr->addr_type.gsi.pkey_ix = umad_get_pkey(umad);
 	osm_mad_addr->addr_type.gsi.service_level = ib_mad_addr->sl;
-	osm_mad_addr->addr_type.gsi.global_route = 0;	/* FIXME: handle GRH */
-	memset(&osm_mad_addr->addr_type.gsi.grh_info, 0,
-	       sizeof osm_mad_addr->addr_type.gsi.grh_info);
+	if (ib_mad_addr->grh_present) {
+		osm_mad_addr->addr_type.gsi.global_route = 1;
+		osm_mad_addr->addr_type.gsi.grh_info.hop_limit = ib_mad_addr->hop_limit;
+		osm_mad_addr->addr_type.gsi.grh_info.ver_class_flow =
+			ib_grh_set_ver_class_flow(6,	/* GRH version */
+						  ib_mad_addr->traffic_class,
+						  ib_mad_addr->flow_label);
+		memcpy(&osm_mad_addr->addr_type.gsi.grh_info.dest_gid,
+		       &ib_mad_addr->gid, 16);
+	}
 }
 
 static void *swap_mad_bufs(osm_madw_t * p_madw, void *umad)
@@ -706,10 +744,14 @@ osm_vendor_open_port(IN osm_vendor_t * const p_vend,
 	for (ca = 0; ca < p_vend->ca_count; ca++) {
 		if ((r = umad_get_ca_portguids(p_vend->ca_names[ca], portguids,
 					       OSM_UMAD_MAX_PORTS_PER_CA + 1)) < 0) {
+#ifdef __WIN__
+			OSM_LOG(p_vend->p_log, OSM_LOG_VERBOSE,
+#else
 			OSM_LOG(p_vend->p_log, OSM_LOG_ERROR, "ERR 5421: "
+#endif
 				"Unable to get CA %s port guids (%s)\n",
 				p_vend->ca_names[ca], strerror(r));
-			goto Exit;
+			continue;
 		}
 		for (i = 0; i < r; i++)
 			if (port_guid == portguids[i]) {
@@ -1069,9 +1111,10 @@ osm_vendor_send(IN osm_bind_handle_t h_bind,
 	osm_mad_addr_t *const p_mad_addr = osm_madw_get_mad_addr_ptr(p_madw);
 	ib_mad_t *const p_mad = osm_madw_get_mad_ptr(p_madw);
 	ib_sa_mad_t *const p_sa = (ib_sa_mad_t *) p_mad;
+	ib_mad_addr_t mad_addr;
 	int ret = -1;
 	int __attribute__((__unused__)) is_rmpp = 0;
-	uint32_t sent_mad_size;
+	uint32_t sent_mad_size, timeout = 0;
 	uint64_t tid;
 #ifndef VENDOR_RMPP_SUPPORT
 	uint32_t paylen = 0;
@@ -1084,12 +1127,12 @@ osm_vendor_send(IN osm_bind_handle_t h_bind,
 
 	if (p_mad->mgmt_class == IB_MCLASS_SUBN_DIR) {
 		umad_set_addr_net(p_vw->umad, 0xffff, 0, 0, 0);
-		umad_set_grh(p_vw->umad, 0);
+		umad_set_grh(p_vw->umad, NULL);
 		goto Resp;
 	}
 	if (p_mad->mgmt_class == IB_MCLASS_SUBN_LID) {
 		umad_set_addr_net(p_vw->umad, p_mad_addr->dest_lid, 0, 0, 0);
-		umad_set_grh(p_vw->umad, 0);
+		umad_set_grh(p_vw->umad, NULL);
 		goto Resp;
 	}
 	/* GS classes */
@@ -1097,9 +1140,19 @@ osm_vendor_send(IN osm_bind_handle_t h_bind,
 			  p_mad_addr->addr_type.gsi.remote_qp,
 			  p_mad_addr->addr_type.gsi.service_level,
 			  IB_QP1_WELL_KNOWN_Q_KEY);
-	umad_set_grh(p_vw->umad, 0);	/* FIXME: GRH support */
+	if (p_mad_addr->addr_type.gsi.global_route) {
+		mad_addr.grh_present = 1;
+		mad_addr.gid_index = 0;
+		mad_addr.hop_limit = p_mad_addr->addr_type.gsi.grh_info.hop_limit;
+		ib_grh_get_ver_class_flow(p_mad_addr->addr_type.gsi.grh_info.ver_class_flow,
+					  NULL, &mad_addr.traffic_class,
+					  &mad_addr.flow_label);
+		memcpy(&mad_addr.gid, &p_mad_addr->addr_type.gsi.grh_info.dest_gid, 16);
+		umad_set_grh(p_vw->umad, &mad_addr);
+	} else
+		umad_set_grh(p_vw->umad, NULL);
 	umad_set_pkey(p_vw->umad, p_mad_addr->addr_type.gsi.pkey_ix);
-	if (ib_class_is_rmpp(p_mad->mgmt_class)) {	/* RMPP GS classes     FIXME: no GRH */
+	if (ib_class_is_rmpp(p_mad->mgmt_class)) {	/* RMPP GS classes */
 		if (!ib_rmpp_is_flag_set((ib_rmpp_mad_t *) p_sa,
 					 IB_RMPP_FLAG_ACTIVE)) {
 			/* Clear RMPP header when RMPP not ACTIVE */
@@ -1138,9 +1191,14 @@ Resp:
 	    p_madw->mad_size;
 #endif
 	tid = cl_ntoh64(p_mad->trans_id);
+	if (resp_expected) {
+		if (p_madw->timeout)
+			timeout = p_madw->timeout;
+		else
+			timeout = p_bind->timeout;
+	}
 	if ((ret = umad_send(p_bind->port_id, p_bind->agent_id, p_vw->umad,
-			     sent_mad_size,
-			     resp_expected ? p_bind->timeout : 0,
+			     sent_mad_size, timeout,
 			     p_bind->max_retries)) < 0) {
 		OSM_LOG(p_vend->p_log, OSM_LOG_ERROR, "ERR 5430: "
 			"Send p_madw = %p of size %d, Class 0x%x, Method 0x%X, "
