@@ -155,10 +155,11 @@ static void create_mgrp_switch_map(cl_qmap_t * m, cl_qlist_t * port_list)
 		if (port->p_node->sw) {
 			sw = port->p_node->sw;
 			sw->is_mc_member = 1;
-		} else {
+		} else if (port->p_physp->p_remote_physp) {
 			sw = port->p_physp->p_remote_physp->p_node->sw;
 			sw->num_of_mcm++;
-		}
+		} else
+			continue;
 		guid = osm_node_get_node_guid(sw->p_node);
 		if (cl_qmap_get(m, guid) == cl_qmap_end(m))
 			cl_qmap_insert(m, guid, &sw->mgrp_item);
@@ -377,7 +378,7 @@ static int mcast_mgr_set_mft_block(osm_sm_t * sm, IN osm_switch_t * p_sw,
 				     IB_MAD_ATTR_MCAST_FWD_TBL,
 				     cl_hton32(block_id_ho), FALSE,
 				     ib_port_info_get_m_key(&p_physp->port_info),
-				     CL_DISP_MSGID_NONE, &context);
+				     0, CL_DISP_MSGID_NONE, &context);
 		if (status != IB_SUCCESS) {
 			OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR 0A02: "
 				"Sending multicast fwd. tbl. block 0x%X to %s "
@@ -1093,7 +1094,7 @@ static void mcast_mgr_set_mfttop(IN osm_sm_t * sm, IN osm_switch_t * p_sw)
 				     sizeof(si), IB_MAD_ATTR_SWITCH_INFO,
 				     0, FALSE,
 				     ib_port_info_get_m_key(&p_physp->port_info),
-				     CL_DISP_MSGID_NONE, &context);
+				     0, CL_DISP_MSGID_NONE, &context);
 
 		if (status != IB_SUCCESS)
 			OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR 0A1B: "
