@@ -283,15 +283,16 @@ void osm_qos_policy_qos_level_destroy(osm_qos_level_t * p)
 	if (!p)
 		return;
 
-	if (p->name)
-		free(p->name);
-	if (p->use)
-		free(p->use);
+	free(p->name);
+	free(p->use);
 
 	for (i = 0; i < p->path_bits_range_len; i++)
 		free(p->path_bits_range_arr[i]);
-	if (p->path_bits_range_arr)
-		free(p->path_bits_range_arr);
+	free(p->path_bits_range_arr);
+
+	for(i = 0; i < p->pkey_range_len; i++)
+		free((p->pkey_range_arr[i]));
+	free(p->pkey_range_arr);
 
 	free(p);
 }
@@ -373,20 +374,23 @@ void osm_qos_policy_match_rule_destroy(osm_qos_match_rule_t * p)
 	if (p->use)
 		free(p->use);
 
-	for (i = 0; i < p->service_id_range_len; i++)
-		free(p->service_id_range_arr[i]);
-	if (p->service_id_range_arr)
+	if (p->service_id_range_arr) {
+		for (i = 0; i < p->service_id_range_len; i++)
+			free(p->service_id_range_arr[i]);
 		free(p->service_id_range_arr);
+	}
 
-	for (i = 0; i < p->qos_class_range_len; i++)
-		free(p->qos_class_range_arr[i]);
-	if (p->qos_class_range_arr)
+	if (p->qos_class_range_arr) {
+		for (i = 0; i < p->qos_class_range_len; i++)
+			free(p->qos_class_range_arr[i]);
 		free(p->qos_class_range_arr);
+	}
 
-	for (i = 0; i < p->pkey_range_len; i++)
-		free(p->pkey_range_arr[i]);
-	if (p->pkey_range_arr)
+	if (p->pkey_range_arr) {
+		for (i = 0; i < p->pkey_range_len; i++)
+			free(p->pkey_range_arr[i]);
 		free(p->pkey_range_arr);
+	}
 
 	cl_list_apply_func(&p->source_list, __free_single_element, NULL);
 	cl_list_remove_all(&p->source_list);
@@ -640,19 +644,19 @@ static osm_qos_match_rule_t *__qos_policy_get_match_rule_by_params(
 		 */
 		if (cl_list_count(&p_qos_match_rule->source_group_list)
 		    && cl_list_count(&p_qos_match_rule->destination_group_list)) {
-			if (!__qos_policy_is_port_in_group_list(p_qos_policy,
-								p_src_physp,
-								&p_qos_match_rule->
-								source_group_list)
-			    && !__qos_policy_is_port_in_group_list(p_qos_policy,
-								   p_dest_physp,
-								   &p_qos_match_rule->
-								   destination_group_list))
-			{
+			if (__qos_policy_is_port_in_group_list(p_qos_policy,
+							       p_src_physp,
+							       &p_qos_match_rule->
+							       source_group_list)
+			    && __qos_policy_is_port_in_group_list(p_qos_policy,
+								  p_dest_physp,
+								  &p_qos_match_rule->
+								  destination_group_list))
+				matched_by_sordguid = TRUE;
+			else {
 				list_iterator = cl_list_next(list_iterator);
 				continue;
 			}
-			matched_by_sordguid = TRUE;
 		}
 
 		/* If a match rule has QoS classes, PR request HAS
@@ -744,7 +748,7 @@ static osm_qos_match_rule_t *__qos_policy_get_match_rule_by_params(
 
 static osm_qos_level_t *__qos_policy_get_qos_level_by_name(
 		const osm_qos_policy_t * p_qos_policy,
-		char *name)
+		const char *name)
 {
 	osm_qos_level_t *p_qos_level = NULL;
 	cl_list_iterator_t list_iterator;
@@ -800,8 +804,6 @@ static void __qos_policy_validate_pkey(
 			osm_qos_match_rule_t * p_qos_match_rule,
 			osm_prtn_t * p_prtn)
 {
-	int i = 0;
-
 	if (!p_qos_policy || !p_qos_match_rule || !p_prtn)
 		return;
 
@@ -809,40 +811,11 @@ static void __qos_policy_validate_pkey(
 	    p_prtn->sl == p_qos_match_rule->p_qos_level->sl)
 		return;
 
-	/* overriding partition's SL */
-	OSM_LOG(&p_qos_policy->p_subn->p_osm->log, OSM_LOG_ERROR,
-		"ERR AC15: pkey 0x%04X in match rule - "
-		"overriding partition SL (%u) with QoS Level SL (%u)\n",
-		cl_ntoh16(p_prtn->pkey), p_prtn->sl,
-		p_qos_match_rule->p_qos_level->sl);
-	p_prtn->sl = p_qos_match_rule->p_qos_level->sl;
-
-
-	/* If this partition has default MCast groups.
-	 * Fix those group's SL too */
-	for (i = 0; i < p_prtn->nmgrps; i++)
-	{
-		uint8_t sl;
-		uint32_t flow;
-		uint8_t hop;
-
-		CL_ASSERT((cl_ntoh16(p_prtn->mgrps[i]->mcmember_rec.pkey) & 0x7fff) ==
-			  (cl_ntoh16(p_prtn->pkey) & 0x7fff));
-
-		ib_member_get_sl_flow_hop(p_prtn->mgrps[i]->mcmember_rec.sl_flow_hop,
-					  &sl, &flow, &hop);
-		if (sl != p_prtn->sl) {
-			char gid_str[INET6_ADDRSTRLEN];
-			OSM_LOG(&p_qos_policy->p_subn->p_osm->log, OSM_LOG_DEBUG,
-				"Updating MCGroup (MGID %s) SL to "
-				"match partition SL (%u)\n",
-				inet_ntop(AF_INET6, p_prtn->mgrps[i]->mcmember_rec.mgid.raw,
-					  gid_str, sizeof gid_str),
-				p_prtn->sl);
-			p_prtn->mgrps[i]->mcmember_rec.sl_flow_hop =
-				ib_member_set_sl_flow_hop(p_prtn->sl, flow, hop);
-		}
-	}
+	OSM_LOG(&p_qos_policy->p_subn->p_osm->log, OSM_LOG_VERBOSE,
+		"QoS Level SL (%u) for Pkey 0x%04X in match rule "
+		"differs from  partition SL (%u)\n",
+		p_qos_match_rule->p_qos_level->sl,
+		cl_ntoh16(p_prtn->pkey), p_prtn->sl);
 }
 
 /***************************************************
