@@ -262,7 +262,7 @@ static int disable_port(osm_sm_t *sm, osm_physp_t *p)
 			   payload, sizeof(payload), IB_MAD_ATTR_PORT_INFO,
 			   cl_hton32(osm_physp_get_port_num(p)),
 			   FALSE, m_key,
-			   CL_DISP_MSGID_NONE, &context);
+			   0, CL_DISP_MSGID_NONE, &context);
 	return status;
 }
 
@@ -338,7 +338,10 @@ static int shutup_noisy_port(osm_sm_t *sm, ib_net16_t lid, uint8_t port,
 			cl_ntoh16(lid), port);
 		if (disable_port(sm, p))
 			OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR 3811: "
-				"Failed to disable.\n");
+				"Failed to disable noisy physical port 0x%016"
+				PRIx64 ": lid %u, num %u\n",
+				cl_ntoh64(osm_physp_get_port_guid(p)),
+				cl_ntoh16(lid), port);
 		else
 			return 1;
 	}
@@ -387,7 +390,7 @@ static void trap_rcv_process_request(IN osm_sm_t * sm,
 		   Otherwise we start a sweep on the trap 144 caused by
 		   cleaning up SM Cap bit...
 		 */
-		goto Exit;
+		goto Exit2;
 
 	/* update the is_gsi flag according to the mgmt_class field */
 	if (p_madw->p_mad->mgmt_class == IB_MCLASS_SUBN_LID ||
@@ -403,7 +406,7 @@ static void trap_rcv_process_request(IN osm_sm_t * sm,
 	if (p_smp->method != IB_MAD_METHOD_TRAP) {
 		OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR 3801: "
 			"Unsupported method 0x%X\n", p_smp->method);
-		goto Exit;
+		goto Exit2;
 	}
 
 	/*
@@ -429,7 +432,7 @@ static void trap_rcv_process_request(IN osm_sm_t * sm,
 			if (sm->p_subn->sm_base_lid == 0) {
 				OSM_LOG(sm->p_log, OSM_LOG_DEBUG,
 					"Received SLID=0 Trap with local LID=0. Ignoring MAD\n");
-				goto Exit;
+				goto Exit2;
 			}
 			OSM_LOG(sm->p_log, OSM_LOG_DEBUG,
 				"Received SLID=0 Trap. Using local LID:%u instead\n",
@@ -662,6 +665,7 @@ check_report:
 
 Exit:
 	CL_PLOCK_RELEASE(sm->p_lock);
+Exit2:
 	OSM_LOG_EXIT(sm->p_log);
 }
 
